import { defineNuxtModule, createResolver, addPlugin, addAutoImport, resolveModule } from '@nuxt/kit';
import { withTrailingSlash } from 'ufo';
import { defu } from 'defu';
import { n as name, v as version, M as MODULE_DEFAULTS, c as createTokensDir, g as generateTokens, r as resolveTokens, l as logger } from './chunks/generate.mjs';
import 'fs';
import 'fs/promises';
import 'browser-style-dictionary/browser.js';
import './formats.mjs';
import 'pathe';
import 'chalk';

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "designTokens",
    compatibility: {
      nuxt: "^3.0.0-rc.5"
    }
  },
  defaults: MODULE_DEFAULTS,
  async setup(moduleOptions, nuxt) {
    nuxt.options.runtimeConfig.designTokens = {
      tokensFilePaths: [],
      server: moduleOptions.server
    };
    const layers = nuxt.options._layers;
    const privateConfig = nuxt.options.runtimeConfig.designTokens;
    const tokensDir = withTrailingSlash(nuxt.options.buildDir + "/tokens");
    globalThis.__nuxtDesignTokensBuildDir__ = tokensDir;
    const { resolve: resolveTokensDir } = createResolver(tokensDir);
    privateConfig.tokensDir = tokensDir;
    const { resolve: resolveRuntime } = createResolver(import.meta.url);
    const resolveRuntimeModule = (path) => resolveModule(path, { paths: resolveRuntime("./runtime") });
    await createTokensDir(tokensDir);
    const refreshTokens = async (nitro) => {
      const { tokensFilePaths, tokens: tokens2 } = resolveTokens(layers);
      if (moduleOptions.tokens) {
        privateConfig.tokensFilePaths = tokensFilePaths;
      }
      if (nitro) {
        await nitro.storage.setItem("cache:design-tokens:tokens.json", tokens2);
      }
      return { tokens: tokens2, tokensFilePaths };
    };
    const buildTokens = async (nitro) => {
      try {
        const start = performance.now();
        const tokens2 = await nitro.storage.getItem("cache:design-tokens:tokens.json");
        await generateTokens(tokens2, tokensDir);
        const end = performance.now();
        const ms = Math.round(end - start);
        logger.success(`Design Tokens built${ms ? " in " + ms + "ms" : ""}`);
      } catch (e) {
        logger.error("Could not build tokens!");
        logger.error(e.message);
      }
    };
    const { tokens } = await refreshTokens();
    nuxt.options.build.transpile = nuxt.options.build.transpile || [];
    nuxt.options.build.transpile.push(resolveRuntime("./runtime"));
    nuxt.options.alias = nuxt.options.alias || {};
    nuxt.options.alias["#design-tokens"] = resolveTokensDir("index");
    nuxt.options.alias["#design-tokens/types"] = resolveTokensDir("types.d");
    nuxt.options.css = nuxt.options.css || [];
    nuxt.options.css = [...nuxt.options.css, resolveTokensDir("tokens.css")];
    nuxt.hook("prepare:types", (opts) => {
      opts.references.push({ path: resolveTokensDir("types.d.ts") });
    });
    nuxt.hook("nitro:build:before", async (nitro) => {
      await refreshTokens(nitro);
      await buildTokens(nitro);
    });
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.handlers = nitroConfig.handlers || [];
      nitroConfig.handlers.push({
        method: "all",
        route: "/api/_design-tokens/tokens",
        handler: resolveRuntimeModule("./server/api/tokens/index")
      });
      nitroConfig.prerender = nitroConfig.prerender || {};
      nitroConfig.prerender.routes = nitroConfig.prerender.routes || [];
      nitroConfig.prerender.routes.push("/api/_design-tokens/tokens");
      nitroConfig.bundledStorage = nitroConfig.bundledStorage || [];
      nitroConfig.bundledStorage.push("/cache/tokens");
      nitroConfig.bundledStorage.push("/tokens");
      if (privateConfig.server) {
        nitroConfig.handlers.push({
          method: "get",
          route: "/api/_design-tokens/tokens/generate",
          handler: resolveRuntimeModule("./server/api/tokens/generate")
        });
        nitroConfig.externals = defu(typeof nitroConfig.externals === "object" ? nitroConfig.externals : {}, {
          inline: [
            "lodash",
            "browser-style-dictionary",
            "browser-style-dictionary/lib",
            "browser-style-dictionary/browser",
            resolveRuntime()
          ]
        });
        nitroConfig.alias["#design-tokens"] = resolveRuntime("./runtime/server/utils");
      }
    });
    addPlugin({
      src: resolveRuntimeModule("./plugins/tokens")
    });
    addAutoImport({
      from: resolveRuntimeModule("./composables/tokens"),
      name: "useTokens",
      as: "useTokens"
    });
    if (nuxt.options.dev) {
      nuxt.hook("nitro:init", (nitro) => {
        nuxt.hook("vite:serverCreated", (viteServer) => {
          nuxt.hook("builder:watch", async (_, path) => {
            const isTokenFile = path.includes("tokens.config.ts") || path.includes("tokens.config.js");
            if (isTokenFile) {
              const { tokens: tokens2 } = await refreshTokens(nitro);
              viteServer.ws.send({
                type: "custom",
                event: "nuxt-design-tokens:update",
                data: {
                  tokens: moduleOptions.tokens ? tokens2 : {}
                }
              });
              await buildTokens(nitro);
            }
          });
        });
      });
    }
    nuxt.hook("build:before", async () => await generateTokens(tokens, tokensDir, true));
    await generateTokens(tokens, tokensDir, true);
    nuxt.hook("tailwindcss:config", (tailwindConfig) => {
      tailwindConfig.content = tailwindConfig.content ?? [];
      tailwindConfig.content.push(`${tokensDir}/**/*`);
    });
  }
});

export { module as default };
